"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProfanitySafeBase60 = '0123456789ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz';
var Identifier = /** @class */ (function () {
    function Identifier(opts) {
        this.buffer = (opts ? opts.buffer : undefined) || Buffer.alloc(0);
        this.alphabet = (opts ? opts.alphabet : undefined) || exports.ProfanitySafeBase60;
        this.minLength = (opts ? opts.minLength : undefined) || 0;
    }
    Identifier.prototype.fromBuffer = function (buffer) {
        this.buffer = buffer;
        return this.trimBuffer();
    };
    Identifier.prototype.trimBuffer = function () {
        if (this.buffer.length < 0 || this.buffer[0] !== 0x00)
            return this;
        for (var i = 0; i <= this.buffer.length; i++)
            if (this.buffer[i] !== 0x00)
                return this.fromBuffer(this.buffer.slice(i));
        this.buffer = Buffer.alloc(0);
        return this;
    };
    Identifier.prototype.toBuffer = function () {
        return this.buffer;
    };
    Identifier.prototype.fromHex = function (hex) {
        this.buffer = Buffer.from(hex, 'hex');
        return this.trimBuffer();
    };
    Identifier.prototype.toHex = function (minLength) {
        var hex = this.buffer.toString('hex');
        return hex.padEnd(minLength || this.minLength, '0');
    };
    Identifier.prototype.fromUniversallyUnique = function (uuid) {
        return this.fromHex(uuid.replace(/-/g, ''));
    };
    Identifier.prototype.toUniversallyUnique = function () {
        var hex = this.toHex().padEnd(32, '0');
        var sections = [];
        sections.push(hex.substr(0, 8));
        sections.push(hex.substr(8, 4));
        sections.push(hex.substr(12, 4));
        sections.push(hex.substr(16, 4));
        sections.push(hex.substr(20));
        return sections.join('-');
    };
    Identifier.prototype.fromString = function (stringInput, alphabet) {
        return this.fromBuffer(require('base-x')(alphabet || this.alphabet).decode(stringInput));
    };
    Identifier.prototype.toString = function (arg1, arg2) {
        var optMinimumLength = typeof arg1 === 'number' ? arg1 : typeof arg2 === 'number' ? arg2 : this.minLength;
        var optAlphabet = typeof arg1 === 'string' ? arg1 : typeof arg2 === 'string' ? arg2 : this.alphabet;
        return require('base-x')(optAlphabet).encode(this.buffer).padStart(optMinimumLength, optAlphabet[0]);
    };
    Identifier.prototype.fromInteger = function (numberInput) {
        return this.fromHex(numberInput.toString(16));
    };
    Identifier.prototype.toInteger = function () {
        return parseInt(this.toHex(), 16);
    };
    Identifier.prototype.fromBigInt = function (bigInt) {
        return this.fromHex(bigInt.toString(16));
    };
    Identifier.prototype.toBigInt = function () {
        return BigInt('0x' + this.toHex());
    };
    Identifier.prototype.generateObject = function () {
        return this.generateHexWithTimestamp(24);
    };
    Identifier.prototype.generateBigInt = function (includeMilliseconds) {
        if (includeMilliseconds === void 0) { includeMilliseconds = false; }
        return this.generateHexWithTimestamp(16, false);
    };
    Identifier.prototype.generateRandomBigInt = function () {
        return this.generateHex(16);
    };
    Identifier.prototype.generateHexWithTimestamp = function (length, includeMilliseconds) {
        if (includeMilliseconds === void 0) { includeMilliseconds = false; }
        var time = this.getTimeAsHex(includeMilliseconds);
        var random = this.getRandomHex(length - time.length);
        return this.fromHex(time + random);
    };
    Identifier.prototype.generateHex = function (length) {
        return this.fromHex(this.getRandomHex(length));
    };
    Identifier.prototype.generateUniversallyUnique = function (processNumber) {
        var process = processNumber ? processNumber.toString(16).padStart(6, '0') : this.getRandomHex(6);
        var processHigh = process.substr(0, 3);
        var processLow = process.substr(3, 3);
        var version = 4;
        var seq = (8 + Math.round(Math.random() * 3)).toString(16);
        var time = this.getTimeAsHex();
        var random = this.getRandomHex(12);
        return this.fromHex(time + version + processHigh + seq + processLow + random);
    };
    Identifier.prototype.generateRandomString = function (desiredLength) {
        var output = '';
        if (desiredLength > 0)
            output = this.getRandomAlphabetChar(false);
        for (var i = 0; i < desiredLength - 2; i++)
            output += this.getRandomAlphabetChar();
        if (desiredLength > 1)
            output += this.getRandomAlphabetChar(false);
        return this.fromString(output);
    };
    Identifier.prototype.getTimeAsHex = function (includeMilliseconds) {
        if (includeMilliseconds === void 0) { includeMilliseconds = true; }
        var time = new Date().getTime();
        var seconds = Math.floor(time / 1000)
            .toString(16)
            .padStart(8, '0');
        if (!includeMilliseconds)
            return seconds;
        var milliseconds = (time % 1000).toString(16).padStart(4, '0');
        return seconds + milliseconds;
    };
    Identifier.prototype.getRandomHexChar = function (allowZero) {
        if (allowZero === void 0) { allowZero = true; }
        var random = Math.round(Math.random() * (allowZero ? 15 : 14));
        return (allowZero ? random : random + 1).toString(16);
    };
    Identifier.prototype.getRandomHex = function (desiredLength) {
        return require('crypto')
            .randomBytes(desiredLength / 2)
            .toString('hex');
    };
    Identifier.prototype.getRandomAlphabetChar = function (allowZero) {
        if (allowZero === void 0) { allowZero = false; }
        var multiplier = this.alphabet.length - (allowZero ? 1 : 2);
        var random = Math.round(Math.random() * multiplier);
        return this.alphabet[allowZero ? random : random + 1];
    };
    return Identifier;
}());
exports.Identifier = Identifier;
function getIdentifier(opts) {
    return new Identifier(opts);
}
exports.getIdentifier = getIdentifier;
// Instance Creation Helpers
function getIdentifierFromInteger(integer) {
    return getIdentifier().fromInteger(integer);
}
exports.getIdentifierFromInteger = getIdentifierFromInteger;
function getIdentifierFromString(str) {
    return getIdentifier().fromString(str);
}
exports.getIdentifierFromString = getIdentifierFromString;
function getIdentifierFromHex(hex) {
    return getIdentifier().fromHex(hex);
}
exports.getIdentifierFromHex = getIdentifierFromHex;
function getIdentifierFromObject(hex) {
    return getIdentifierFromHex(hex);
}
exports.getIdentifierFromObject = getIdentifierFromObject;
function getIdentifierFromUniversallyUnique(uuid) {
    return getIdentifier().fromUniversallyUnique(uuid);
}
exports.getIdentifierFromUniversallyUnique = getIdentifierFromUniversallyUnique;
// Generation Helpers
function generateObjectIdentifier() {
    return getIdentifier().generateObject();
}
exports.generateObjectIdentifier = generateObjectIdentifier;
function generateUniversallyUniqueIdentifier() {
    return getIdentifier().generateUniversallyUnique();
}
exports.generateUniversallyUniqueIdentifier = generateUniversallyUniqueIdentifier;
function generateIdentifierWithBigInt() {
    return getIdentifier().generateBigInt();
}
exports.generateIdentifierWithBigInt = generateIdentifierWithBigInt;
// String <> Hex Helpers
function getHexIdentifierAsString(id) {
    return getIdentifier().fromHex(id).toString();
}
exports.getHexIdentifierAsString = getHexIdentifierAsString;
function getStringIdentifierAsHex(id) {
    return getIdentifier().fromString(id).toHex();
}
exports.getStringIdentifierAsHex = getStringIdentifierAsHex;
// String <> Integer Helpers
function getIntegerIdentifierAsString(integer) {
    return getIdentifier().fromInteger(integer).toString();
}
exports.getIntegerIdentifierAsString = getIntegerIdentifierAsString;
function getStringIdentifierAsInteger(str) {
    return getIdentifier().fromString(str).toInteger();
}
exports.getStringIdentifierAsInteger = getStringIdentifierAsInteger;
// Hex <> Integer Helpers
function getIntegerIdentifierAsHex(integer) {
    return getIdentifier().fromInteger(integer).toHex();
}
exports.getIntegerIdentifierAsHex = getIntegerIdentifierAsHex;
function getHexIdentifierAsInteger(hex) {
    return getIdentifier().fromHex(hex).toInteger();
}
exports.getHexIdentifierAsInteger = getHexIdentifierAsInteger;
// UUID <> String Helpers
function getUniversallyUniqueIdentifierAsString(id) {
    return getIdentifier().fromUniversallyUnique(id).toString();
}
exports.getUniversallyUniqueIdentifierAsString = getUniversallyUniqueIdentifierAsString;
function getStringIdentifierAsUniversallyUnique(id) {
    return getIdentifier().fromString(id).toUniversallyUnique();
}
exports.getStringIdentifierAsUniversallyUnique = getStringIdentifierAsUniversallyUnique;
